@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2009-2018, VU University Amsterdam
+    Copyright (c)  2009-2022, VU University Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -43,17 +44,12 @@
 #include <wchar.h>
 #include <wctype.h>
 #include <assert.h>
+#include "utf8.h"
 
 static size_t removed_dot_segments(size_t len, const pl_wchar_t *in,
 				   pl_wchar_t *out);
 static pl_wchar_t *remove_last_segment(const pl_wchar_t *base,
 				       const pl_wchar_t *o);
-static char *_utf8_put_char(char *out, int chr);
-
-#define ISUTF8_MB(c) ((unsigned)(c) >= 0xc0 && (unsigned)(c) <= 0xfd)
-#define utf8_put_char(out, chr) \
-	((chr) < 0x80 ? out[0]=(char)(chr), out+1 \
-		      : _utf8_put_char(out, (chr)))
 
 
 		 /*******************************
@@ -1262,8 +1258,7 @@ ranges_in_charbuf(charbuf *cb, uri_component_ranges *ranges)
 
 
 typedef struct
-{ atom_t	       atom;
-  pl_wchar_t	      *text;
+{ pl_wchar_t	      *text;
   uri_component_ranges ranges;
 } base_cache;
 
@@ -1278,10 +1273,8 @@ free_base_cache(void *cache)
   if ( PL_query(PL_QUERY_HALTING) )
     return;
 
-  if ( base->atom )
-  { PL_unregister_atom(base->atom);
+  if ( base->text )
     PL_free(base->text);
-  }
 
   PL_free(base);
 }
@@ -1307,34 +1300,24 @@ static base_cache base_store;
 
 static const uri_component_ranges *
 base_ranges(term_t t)
-{ atom_t a;
+{ size_t len;
+  pl_wchar_t *s;
 
-  if ( PL_get_atom(t, &a) )
+  if ( PL_get_wchars(t, &len, &s, CVT_ATOM|CVT_STRING|BUF_MALLOC|CVT_EXCEPTION) )
   { base_cache *base = myBase();
 
-    if ( base->atom != a )
-    { size_t len;
-      pl_wchar_t *s;
-
-      if ( base->atom )
-      { PL_unregister_atom(base->atom);
-	PL_free(base->text);
-	base->atom = 0;
+    if ( !base->text || wcscmp(base->text, s) != 0 )
+    { if ( base->text )
+      { PL_free(base->text);
 	base->text = NULL;
       }
-      if ( !PL_get_wchars(t, &len, &s, CVT_ATOM|BUF_MALLOC|CVT_EXCEPTION) )
-	return NULL;
-      base->atom = a;
-      PL_register_atom(a);
       base->text = s;
       parse_uri(&base->ranges, len, s);
     }
 
     return &base->ranges;
   } else
-  { type_error("atom", t);
-    return NULL;
-  }
+    return FALSE;
 }
 
 
@@ -1543,46 +1526,6 @@ removed_dot_segments(size_t len, const pl_wchar_t *in, pl_wchar_t *out)
 		 *	    IRI HANDLING	*
 		 *******************************/
 
-#define utf8_put_char(out, chr) \
-	((chr) < 0x80 ? out[0]=(char)(chr), out+1 \
-		      : _utf8_put_char(out, (chr)))
-
-
-static char *
-_utf8_put_char(char *out, int chr)
-{ if ( chr < 0x80 )
-  { *out++ = chr;
-  } else if ( chr < 0x800 )
-  { *out++ = 0xc0|((chr>>6)&0x1f);
-    *out++ = 0x80|(chr&0x3f);
-  } else if ( chr < 0x10000 )
-  { *out++ = 0xe0|((chr>>12)&0x0f);
-    *out++ = 0x80|((chr>>6)&0x3f);
-    *out++ = 0x80|(chr&0x3f);
-  } else if ( chr < 0x200000 )
-  { *out++ = 0xf0|((chr>>18)&0x07);
-    *out++ = 0x80|((chr>>12)&0x3f);
-    *out++ = 0x80|((chr>>6)&0x3f);
-    *out++ = 0x80|(chr&0x3f);
-  } else if ( chr < 0x4000000 )
-  { *out++ = 0xf8|((chr>>24)&0x03);
-    *out++ = 0x80|((chr>>18)&0x3f);
-    *out++ = 0x80|((chr>>12)&0x3f);
-    *out++ = 0x80|((chr>>6)&0x3f);
-    *out++ = 0x80|(chr&0x3f);
-  } else if ( (unsigned)chr < 0x80000000 )
-  { *out++ = 0xfc|((chr>>30)&0x01);
-    *out++ = 0x80|((chr>>24)&0x3f);
-    *out++ = 0x80|((chr>>18)&0x3f);
-    *out++ = 0x80|((chr>>12)&0x3f);
-    *out++ = 0x80|((chr>>6)&0x3f);
-    *out++ = 0x80|(chr&0x3f);
-  }
-
-  return out;
-}
-
-
 /** uri_iri(+URI, -IRI) is det.
     uri_iri(-URI, +IRI) is det.
 
