@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2013-2020, University of Amsterdam
-                              VU University Amsterdam
-                              CWI, Amsterdam
+    Copyright (c)  2013-2022, University of Amsterdam
+			      VU University Amsterdam
+			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -35,8 +36,8 @@
 */
 
 :- module(test_ssl,
-          [ test_ssl/0
-          ]).
+	  [ test_ssl/0
+	  ]).
 
 :- use_module(library(plunit)).
 :- use_module(library(ssl)).
@@ -45,7 +46,9 @@
 :- use_module(library(error)).
 :- use_module(library(readutil)).
 :- use_module(library(socket)).
+:- if(current_prolog_flag(threads, true)).
 :- use_module(https).
+:- endif.
 
 %:- debug(connection).
 %:- debug(certificate).
@@ -54,18 +57,18 @@
 
 test_ssl :-
     run_tests([ ssl_options,
-                ssl_server,
-                ssl_keys,
-                https_open,
-                ssl_certificates,
-                crypto_data_encrypt
-              ]).
+		ssl_server,
+		ssl_keys,
+		https_open,
+		ssl_certificates,
+		crypto_data_encrypt
+	      ]).
 :- dynamic
     option/1,                       % Options to test
     copy_error/1.
 
 run_network_tests :-
-    \+ getenv('USE_PUBLIC_NETWORK_TESTS', false).
+    getenv('SWIPL_PUBLIC_NETWORK_TESTS', true).
 
 %!  cert_file(+File, -Absolute)
 %
@@ -80,10 +83,19 @@ cert_file(File, Abs) :-
 
 test(readme, Title == "# SWI-Prolog SSL interface") :-
     http_download('https://raw.githubusercontent.com\c
-                      /SWI-Prolog/packages-ssl/master/README.md',
-                  String),
+		      /SWI-Prolog/packages-ssl/master/README.md',
+		  String),
     split_string(String, "\n", " \t", [Title|_]).
 
+:- if(\+current_predicate(http_download/2)).
+:- use_module(library(http/http_open)).
+http_download(URL, String) :-
+    http_open(URL, In, []),
+    call_cleanup(
+	read_string(In, _, String),
+	close(In)).
+:- endif.
+
 :- end_tests(https_open).
 
 :- begin_tests(ssl_keys).
@@ -99,9 +111,9 @@ public key for encryption and decryption of messages.
 
 from_file(File, Stream, Goal) :-
     setup_call_cleanup(
-        open(File, read, Stream, [type(binary)]),
-        Goal,
-        close(Stream)).
+	open(File, read, Stream, [type(binary)]),
+	Goal,
+	close(Stream)).
 
 %!  skip_to_pem_cert(+Stream) is det.
 %
@@ -114,27 +126,27 @@ skip_to_pem_cert(In) :-
     (   peek_char(In, '-')
     ->  !
     ;   skip(In, 0'\n),  %'
-        at_end_of_stream(In), !
+	at_end_of_stream(In), !
     ).
 
 test(private_key) :-
     from_file('tests/test_certs/server-key.pem', In,
-              load_private_key(In, "apenoot1", Key)),
+	      load_private_key(In, "apenoot1", Key)),
     is_private_key(Key).
 test(certificate, true) :-
     from_file('tests/test_certs/server-cert.pem', In,
-              ( skip_to_pem_cert(In),
-                load_certificate(In, Cert)
-              )),
+	      ( skip_to_pem_cert(In),
+		load_certificate(In, Cert)
+	      )),
     assertion(is_certificate(Cert)).
 test(trip_private_public, In == Out) :-
     In = "Hello World!",
     from_file('tests/test_certs/server-key.pem', S1,
-              load_private_key(S1, "apenoot1", PrivateKey)),
+	      load_private_key(S1, "apenoot1", PrivateKey)),
     from_file('tests/test_certs/server-cert.pem', S2,
-              ( skip_to_pem_cert(S2),
-                load_certificate(S2, Cert)
-              )),
+	      ( skip_to_pem_cert(S2),
+		load_certificate(S2, Cert)
+	      )),
     certificate_field(Cert, public_key(PublicKey)),
     rsa_private_encrypt(PrivateKey, In, Encrypted, []),
     rsa_public_decrypt(PublicKey, Encrypted, Out, []).
@@ -142,22 +154,22 @@ test(trip_private_public, In == Out) :-
     numlist(1040, 1060, L),
     string_codes(In, L),
     from_file('tests/test_certs/server-key.pem', S1,
-              load_private_key(S1, "apenoot1", PrivateKey)),
+	      load_private_key(S1, "apenoot1", PrivateKey)),
     from_file('tests/test_certs/server-cert.pem', S2,
-              ( skip_to_pem_cert(S2),
-                load_certificate(S2, Cert)
-              )),
+	      ( skip_to_pem_cert(S2),
+		load_certificate(S2, Cert)
+	      )),
     certificate_field(Cert, public_key(PublicKey)),
     rsa_private_encrypt(PrivateKey, In, Encrypted, []),
     rsa_public_decrypt(PublicKey, Encrypted, Out, []).
 test(trip_public_private, In == Out) :-
     In = "Hello World!",
     from_file('tests/test_certs/server-key.pem', S1,
-              load_private_key(S1, "apenoot1", PrivateKey)),
+	      load_private_key(S1, "apenoot1", PrivateKey)),
     from_file('tests/test_certs/server-cert.pem', S2,
-              ( skip_to_pem_cert(S2),
-                load_certificate(S2, Cert)
-              )),
+	      ( skip_to_pem_cert(S2),
+		load_certificate(S2, Cert)
+	      )),
     certificate_field(Cert, public_key(PublicKey)),
     rsa_public_encrypt(PublicKey, In, Encrypted, []),
     rsa_private_decrypt(PrivateKey, Encrypted, Out, []).
@@ -182,35 +194,35 @@ test(cert_mismatch, Msg == 'key values mismatch') :-
     cert_file('etc/server/server-cert.pem', CertFile),
     cert_file('etc/client/client-key.pem', KeyFile),
     options_errmsg([certificate_file(CertFile),
-                    key_file(KeyFile),
-                    password(apenoot2)], Msg).
+		    key_file(KeyFile),
+		    password(apenoot2)], Msg).
 test(cert_mismatch, Msg == 'key values mismatch') :-
     cert_file('etc/server/server-key.pem', KeyFile),
     cert_file('etc/client/client-cert.pem', CertFile),
     read_file_to_string(KeyFile, Key, []),
     read_file_to_string(CertFile, Cert, []),
     options_errmsg([certificate_key_pairs([Cert-Key]),
-                    password('apenoot1')], Msg).
+		    password('apenoot1')], Msg).
 test(cert_mismatch, Msg == 'key values mismatch') :-
     cert_file('etc/server/server-key.pem', KeyFile),
     cert_file('etc/client/client-cert.pem', CertFile),
     read_file_to_string(KeyFile, Key, []),
     read_file_to_string(CertFile, Cert, []),
     options_errmsg([certificate_key_pairs([Cert-Key]),
-                    password('apenoot1'),
-                    sni_hook(sni)], Msg).
+		    password('apenoot1'),
+		    sni_hook(sni)], Msg).
 
 % missing certificate (key specified, with and without sni)
 
 test(missing_cert, Msg == 'no certificate assigned') :-
     cert_file('etc/server/server-key.pem', KeyFile),
     options_errmsg([key_file(KeyFile),
-                    password(apenoot1)], Msg).
+		    password(apenoot1)], Msg).
 test(missing_cert, Msg == 'no certificate assigned') :-
     cert_file('etc/server/server-key.pem', KeyFile),
     options_errmsg([key_file(KeyFile),
-                    password(apenoot1),
-                    sni_hook(sni)], Msg).
+		    password(apenoot1),
+		    sni_hook(sni)], Msg).
 
 % missing key (certificate specified, with and without sni)
 
@@ -220,11 +232,11 @@ test(missing_key, Msg == 'no private key assigned') :-
 test(missing_key, Msg == 'no private key assigned') :-
     cert_file('etc/server/server-cert.pem', CertFile),
     options_errmsg([certificate_file(CertFile),
-                    sni_hook(sni)], Msg).
+		    sni_hook(sni)], Msg).
 
 :- end_tests(ssl_options).
 
-:- begin_tests(ssl_server).
+:- begin_tests(ssl_server, [condition(current_prolog_flag(threads, true))]).
 
 % This test creates a server and client and tests the following
 % sequence:
@@ -244,15 +256,15 @@ test(server) :-
     thread_create(server_loop(SSL, Socket), Id, []),
     (   catch(client, E, true)
     ->  (   var(E)
-        ->  thread_join(Id, Status),
-            report_join_status(Status)
-        ;   format(user_error, 'Client error:~n', []),
-            print_message(error, E),
-            thread_signal(Id, abort),
-            thread_join(Id, Status),
-            report_join_status(Status),
-            fail
-        )
+	->  thread_join(Id, Status),
+	    report_join_status(Status)
+	;   format(user_error, 'Client error:~n', []),
+	    print_message(error, E),
+	    thread_signal(Id, abort),
+	    thread_join(Id, Status),
+	    report_join_status(Status),
+	    fail
+	)
     ).
 
 report_join_status(true).
@@ -264,9 +276,9 @@ report_join_status(exception(Term)) :-
 
 test_ssl(N) :-
     (   between(1, N, _),
-        test_ssl,
-        put('.'), flush_output,
-        fail
+	test_ssl,
+	put('.'), flush_output,
+	fail
     ;   true
     ).
 
@@ -274,9 +286,9 @@ ssl_server :-
     make_server(SSL, Socket),
     server_loop(SSL, Socket).
 
-                 /*******************************
-                 *             SERVER           *
-                 *******************************/
+		 /*******************************
+		 *             SERVER           *
+		 *******************************/
 
 :- dynamic
     stop_server/0,
@@ -284,13 +296,13 @@ ssl_server :-
 
 make_server(SSL, Socket) :-
     ssl_context(server, SSL,
-                [ peer_cert(true),
-                  cacerts([file('tests/test_certs/rootCA/cacert.pem')]),
-                  certificate_file('tests/test_certs/server-cert.pem'),
-                  key_file('tests/test_certs/server-key.pem'),
-                  cert_verify_hook(get_cert_verify),
-                  pem_password_hook(get_server_pwd)
-                ]),
+		[ peer_cert(true),
+		  cacerts([file('tests/test_certs/rootCA/cacert.pem')]),
+		  certificate_file('tests/test_certs/server-cert.pem'),
+		  key_file('tests/test_certs/server-key.pem'),
+		  cert_verify_hook(get_cert_verify),
+		  pem_password_hook(get_server_pwd)
+		]),
     tcp_socket(Socket),
     tcp_setopt(Socket, reuseaddr),
     tcp_bind(Socket, localhost:Port),
@@ -308,7 +320,7 @@ server_loop(SSL, Server) :-
     ;   true
     ),
     catch(copy_client(In, Out), E,
-          assert(copy_error(E))),
+	  assert(copy_error(E))),
     call_cleanup(close(In), close(Out)),
     (   retract(stop_server)
     ->  tcp_close_socket(Server)
@@ -320,50 +332,50 @@ copy_client(In, Out) :-
     (   Line == end_of_file
     ->  true
     ;   debug(data, 'SERVER: Got ~s (sleeping 1.5 sec)', [Line]),
-        sleep(1.5),
-        debug(data, 'SERVER: writing ~s', [Line]),
-        format(Out, '~s~n', [Line]),
-        flush_output(Out),
-        (   atom_codes(bye, Line)
-        ->  assert(stop_server)
-        ;   true
-        ),
-        copy_client(In, Out)
+	sleep(1.5),
+	debug(data, 'SERVER: writing ~s', [Line]),
+	format(Out, '~s~n', [Line]),
+	flush_output(Out),
+	(   atom_codes(bye, Line)
+	->  assert(stop_server)
+	;   true
+	),
+	copy_client(In, Out)
     ).
 
 get_server_pwd(_SSL, "apenoot1") :-
     debug(passwd, 'Returning password from server passwd hook', []).
 
 get_cert_verify(SSL,
-                ProblemCertificate, AllCertificates, FirstCertificate,
-                Error) :-
+		ProblemCertificate, AllCertificates, FirstCertificate,
+		Error) :-
     debug(certificate,
-          'Accept from ~p, \c
-               ProblemCert: ~p, AllCerts: ~p, FirstCert: ~p, \c
-               Error: ~p',
-          [ SSL,
-            ProblemCertificate, AllCertificates, FirstCertificate,
-            Error
-          ]),
+	  'Accept from ~p, \c
+	       ProblemCert: ~p, AllCerts: ~p, FirstCert: ~p, \c
+	       Error: ~p',
+	  [ SSL,
+	    ProblemCertificate, AllCertificates, FirstCertificate,
+	    Error
+	  ]),
     (   Error == verified
     ->  true
     ;   domain_error(verified, Error)
     ).
 
 
-                 /*******************************
-                 *             CLIENT           *
-                 *******************************/
+		 /*******************************
+		 *             CLIENT           *
+		 *******************************/
 
 client :-
     ssl_context(client, SSL,
-             [ host('localhost'),
-               cacerts([file('tests/test_certs/rootCA/cacert.pem')]),
-               certificate_file('tests/test_certs/client-cert.pem'),
-               key_file('tests/test_certs/client-key.pem'),
-               close_parent(true),
-               pem_password_hook(get_client_pwd)
-             ]),
+	     [ host('localhost'),
+	       cacerts([file('tests/test_certs/rootCA/cacert.pem')]),
+	       certificate_file('tests/test_certs/client-cert.pem'),
+	       key_file('tests/test_certs/client-key.pem'),
+	       close_parent(true),
+	       pem_password_hook(get_client_pwd)
+	     ]),
     client_loop(SSL).
 
 client_loop(SSL) :-
@@ -376,7 +388,7 @@ client_loop(SSL) :-
     write_server(Message, In, Out),
     (   option(timeout(T))
     ->  Wait is T*2,
-        sleep(Wait)
+	sleep(Wait)
     ;   true
     ),
     write_server(bye, In, Out),
@@ -389,12 +401,12 @@ write_server(Message, In, Out) :-
     sleep(0.1),
     debug(data, 'CLIENT: attempting to read reply (timeout 1 sec)', []),
     catch(read_from_server(In, Message),
-          E,
-          debug(data, 'CLIENT: exception: ~q', [E])),
+	  E,
+	  debug(data, 'CLIENT: exception: ~q', [E])),
     (   var(E)
     ->  true
     ;   debug(data, 'CLIENT: retrying read reply (timeout 1 sec)', []),
-        read_from_server(In, Message)
+	read_from_server(In, Message)
     ).
 
 read_from_server(In, Message) :-
@@ -402,12 +414,12 @@ read_from_server(In, Message) :-
     (   Line == end_of_file
     ->  true
     ;   atom_codes(Reply, Line),
-        debug(data, 'CLIENT: Got ~q', [Reply]),
-        (   Reply == Message
-        ->  true
-        ;   format(user_error, 'CLIENT: ERROR: Sent ~q, Got ~q~n',
-                   [Message, Reply])
-        )
+	debug(data, 'CLIENT: Got ~q', [Reply]),
+	(   Reply == Message
+	->  true
+	;   format(user_error, 'CLIENT: ERROR: Sent ~q, Got ~q~n',
+		   [Message, Reply])
+	)
     ).
 
 get_client_pwd(_SSL, "apenoot2") :-
@@ -415,11 +427,11 @@ get_client_pwd(_SSL, "apenoot2") :-
 
 :- end_tests(ssl_server).
 
-                 /*******************************
-                 *             CERTS            *
-                 *******************************/
+		 /*******************************
+		 *             CERTS            *
+		 *******************************/
 
-:- begin_tests(ssl_certificates).
+:- begin_tests(ssl_certificates, [condition(current_prolog_flag(threads,true))]).
 
 :- dynamic
     certificate_verification_result/1,
@@ -439,43 +451,44 @@ do_verification_test(Key, Goal, VerificationResults, Status) :-
     retractall(test_port(_)),
     asserta(test_port(Port)),
     (   setup_call_cleanup(
-            thread_create(verification_server(Key, ServerFd, Id), Id, []),
-            catch(Goal,
-                  Exception,
-                  Status = error(Exception)),
-            stop_verification_server(Id))
+	    thread_create(verification_server(Key, ServerFd, Id), Id, []),
+	    catch(Goal,
+		  Exception,
+		  Status = error(Exception)),
+	    stop_verification_server(Id))
     ->  ignore(Status = true)
     ;   Status = fail
     ),
     findall(VerificationResult,
-            retract(certificate_verification_result(VerificationResult)),
-            VerificationResults).
+	    retract(certificate_verification_result(VerificationResult)),
+	    VerificationResults0),
+    sort(VerificationResults0, VerificationResults).
 
 stop_verification_server(Id):-
     assert(stop_server(Id)),
     tcp_socket(S),
     test_port(Port),
     catch((tcp_connect(S, localhost:Port, Read, Write),
-           close(Write, [force(true)]),
-           close(Read, [force(true)])),
-          _,
-          tcp_close_socket(S)),
+	   close(Write, [force(true)]),
+	   close(Read, [force(true)])),
+	  _,
+	  tcp_close_socket(S)),
     thread_join(Id, _Status).
 
 verification_server(TestKey, ServerFd, Id):-
     setup_call_cleanup(true,
-                       verification_server_1(TestKey, Id, ServerFd),
-                       tcp_close_socket(ServerFd)).
+		       verification_server_1(TestKey, Id, ServerFd),
+		       tcp_close_socket(ServerFd)).
 
 verification_server_1(TestKey, Id, ServerFd):-
     tcp_listen(ServerFd, 5),
     format(atom(Key), 'tests/test_certs/~w-key.pem', [TestKey]),
     format(atom(Cert), 'tests/test_certs/~w-cert.pem', [TestKey]),
     ssl_context(server, SSL,
-                [ certificate_file(Cert),
-                  key_file(Key),
-                  password("apenoot")
-                ]),
+		[ certificate_file(Cert),
+		  key_file(Key),
+		  password("apenoot")
+		]),
     verification_server_loop(Id, ServerFd, SSL).
 
 verification_server_loop(Id, _ServerFd, _SSL) :-
@@ -484,17 +497,17 @@ verification_server_loop(Id, _ServerFd, _SSL) :-
 
 verification_server_loop(Id, ServerFd, SSL) :-
     catch(accept_client(ServerFd, SSL),
-          _Term,
-          true),
+	  _Term,
+	  true),
     verification_server_loop(Id, ServerFd, SSL).
 
 accept_client(ServerFd, SSL):-
     tcp_accept(ServerFd, ClientFd, _Peer),
     setup_call_cleanup(tcp_open_socket(ClientFd, PlainIn, PlainOut),
-                       dispatch_client(SSL, PlainIn, PlainOut),
-                       ( close(PlainOut, [force(true)]),
-                         close(PlainIn, [force(true)])
-                       )).
+		       dispatch_client(SSL, PlainIn, PlainOut),
+		       ( close(PlainOut, [force(true)]),
+			 close(PlainIn, [force(true)])
+		       )).
 
 
 dispatch_client(SSL, PlainIn, PlainOut):-
@@ -515,139 +528,147 @@ try_ssl_client(Hostname, Hook):-
 try_ssl_client(Hostname, Hook, Options):-
     test_port(Port),
     ssl_context(client, SSL,
-                [ host(Hostname),
-                  port(Port),
-                  cert_verify_hook(Hook),
-                  cacerts([file('tests/test_certs/rootCA/cacert.pem')])
-                | Options
-                ]),
+		[ host(Hostname),
+		  port(Port),
+		  cert_verify_hook(Hook),
+		  cacerts([file('tests/test_certs/rootCA/cacert.pem')])
+		| Options
+		]),
     % Always connect to localhost
     verify_client(localhost:Port, SSL).
 
 verify_client(Address, SSL) :-
     tcp_socket(S),
     setup_call_cleanup(tcp_connect(S, Address, PlainIn, PlainOut),
-                       verify_client_1(SSL, PlainIn, PlainOut),
-                       ( close(PlainOut, [force(true)]),
-                         close(PlainIn, [force(true)])
-                       )).
+		       verify_client_1(SSL, PlainIn, PlainOut),
+		       ( close(PlainOut, [force(true)]),
+			 close(PlainIn, [force(true)])
+		       )).
 
 verify_client_1(SSL, PlainIn, PlainOut):-
     set_stream(PlainIn, timeout(1)),
     setup_call_cleanup(ssl_negotiate(SSL, PlainIn, PlainOut, SSLIn, SSLOut),
-                       ( format(SSLOut, 'Hello~n', []),
-                         flush_output(SSLOut)
-                       ),
-                       ( close(SSLOut, [force(true)]),
-                         close(SSLIn, [force(true)])
-                       )).
+		       ( format(SSLOut, 'Hello~n', []),
+			 flush_output(SSLOut),
+			 read_line_to_codes(SSLIn, Codes),
+			 assertion(Codes == `Hello`)
+		       ),
+		       ( close(SSLOut, [force(true)]),
+			 close(SSLIn, [force(true)])
+		       )).
 
 zz(Goal):-
     setup_call_catcher_cleanup(format('CALL : ~q~n', [Goal]),
-                               Goal,
-                               Catcher,
-                               ( Catcher = exception(E) -> format('ERROR: ~q (~q)~n', [Goal, E])
-                               ; Catcher == fail-> format('FAIL : ~q~n', [Goal])
-                               ; format('EXIT : ~q~n', [Goal]))
-                              ),
+			       Goal,
+			       Catcher,
+			       ( Catcher = exception(E) -> format('ERROR: ~q (~q)~n', [Goal, E])
+			       ; Catcher == fail-> format('FAIL : ~q~n', [Goal])
+			       ; format('EXIT : ~q~n', [Goal]))
+			      ),
     ( var(Catcher)->
-        format('PEND : ~q~n', [Goal])
+	format('PEND : ~q~n', [Goal])
     ; true
     ).
 
 
 test_verify_hook(_,_,_,_,Error):-
-    assert(certificate_verification_result(Error)).
+    (   certificate_verification_result(Error)
+    ->  true    % pre 1.1.1h we could get two calls claiming `verified`.
+    ;   assertz(certificate_verification_result(Error))
+    ).
 
 fail_verify_hook(_,_,_,_,Error):-
     Error == verified.
 
 abort_verify_hook(_,_,_,_,Error):-
     ( Error == verified->
-        true
+	true
     ; throw(error(certificate_error(Error), _))
     ).
 
 test_crl_hook(_, _, _, _, verified).
 test_crl_hook(_SSL, Cert, _Chain, _Tail, revoked):-
     setup_call_cleanup(open('tests/test_certs/rootCA-crl.pem', read, Stream),
-                       load_crl(Stream, CRL),
-                       close(Stream)),
+		       load_crl(Stream, CRL),
+		       close(Stream)),
     certificate_field(Cert, serial(Serial)),
     memberchk(revocations(Revocations), CRL),
     \+memberchk(revoked(Serial, _RevocationTime), Revocations).
 
 
-test('Valid certificate, correct hostname in CN, signed by trusted CA', VerificationResults:Status == [verified, verified]:true):-
+test('Valid certificate, correct hostname in CN, signed by trusted CA', VerificationResults:Status == [verified]:true):-
     do_verification_test(1, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Valid certificate, correct hostname in SAN, signed by trusted CA', VerificationResults:Status == [verified, verified]:true):-
+test('Valid certificate, correct hostname in SAN, signed by trusted CA', VerificationResults:Status == [verified]:true):-
     do_verification_test(2, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Valid certificate, incorrect hostname in CN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
+test('Valid certificate, incorrect hostname in CN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified]:true):-
     do_verification_test(3, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Valid certificate, incorrect hostname in SAN and CN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
+test('Valid certificate, incorrect hostname in SAN and CN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified]:true):-
     do_verification_test(4, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Valid certificate, correct wildcard hostname in SAN, signed by trusted CA', VerificationResults:Status == [verified, verified]:true):-
+test('Valid certificate, correct wildcard hostname in SAN, signed by trusted CA', VerificationResults:Status == [verified]:true):-
     do_verification_test(5, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Valid certificate, incorrect wildcard hostname in SAN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
+test('Valid certificate, incorrect wildcard hostname in SAN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified]:true):-
     do_verification_test(6, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Valid certificate, wildcard hostname in SAN with wildcard too high, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
+test('Valid certificate, wildcard hostname in SAN with wildcard too high, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified]:true):-
     do_verification_test(7, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Valid certificate, wildcard hostname in SAN with wildcard too low, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
+test('Valid certificate, wildcard hostname in SAN with wildcard too low, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified]:true):-
     do_verification_test(8, try_ssl_client('www.bad.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Valid certificate, wildcard hostname in SAN with wildcard in right level of domain, signed by trusted CA', VerificationResults:Status == [verified, verified]:true):-
+test('Valid certificate, wildcard hostname in SAN with wildcard in right level of domain, signed by trusted CA', VerificationResults:Status == [verified]:true):-
     do_verification_test(9, try_ssl_client('www.good.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Valid certificate, illegal wildcard hostname in CN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
+test('Valid certificate, illegal wildcard hostname in CN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified]:true):-
     do_verification_test(10, try_ssl_client('www.good.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Hostname containing embedded NULL, signed by trusted CA',
      [ true(VerificationResults:Status ==
-            [hostname_mismatch,verified,verified]:true),
+	    [hostname_mismatch,verified]:true),
        condition((size_file('tests/test_certs/11-cert.pem', Size),
-                  Size > 0))
+		  Size > 0))
      ]):-
     do_verification_test(11, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Certificate which has expired, signed by trusted CA', VerificationResults:Status == [verified, expired, verified]:true):-
+test('Certificate which has expired, signed by trusted CA', VerificationResults:Status == [expired, verified]:true):-
     do_verification_test(12, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Certificate which is not yet valid, signed by trusted CA', VerificationResults:Status == [verified, not_yet_valid, verified]:true):-
+test('Certificate which is not yet valid, signed by trusted CA', VerificationResults:Status == [not_yet_valid, verified]:true):-
     do_verification_test(13, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Certificate is not issued by trusted CA'):-
     do_verification_test(14, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status),
-    ( VerificationResults:Status == [unknown_issuer, unknown_issuer]:true ->
-        % OpenSSL 1.0.2 and above
-        true
-    ; VerificationResults:Status == [unknown_issuer, not_trusted, unknown_issuer]:true ->
-        % OpenSSL 1.0.1 and below
-        true
+    ( VerificationResults:Status == [unknown_issuer]:true ->
+	% OpenSSL 1.0.2 - 1.1.1h
+	true
+    ; VerificationResults:Status == [unknown_issuer, not_trusted]:true ->
+	% OpenSSL 1.0.1 and below
+	true
+    ; VerificationResults:Status == [unknown_issuer, verified]:true ->
+	% OpenSSL 1.1.1i and above
+	true
     ).
 
-test('Certificate is issued by trusted CA but has been altered so signature is wrong', VerificationResults:Status == [verified, bad_signature, verified]:true):-
+test('Certificate is issued by trusted CA but has been altered so signature is wrong', VerificationResults:Status == [bad_signature, verified]:true):-
     do_verification_test(15, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Certificate is not intended for SSL', VerificationResults:Status == [bad_certificate_use, verified, verified]:true):-
+test('Certificate is not intended for SSL', VerificationResults:Status == [bad_certificate_use, verified]:true):-
     do_verification_test(17, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Certificate signed not-explicitly-trusted intermediary requiring us to follow the chain', VerificationResults:Status == [verified, verified, verified]:true):-
+test('Certificate signed not-explicitly-trusted intermediary requiring us to follow the chain', VerificationResults:Status == [verified]:true):-
     do_verification_test(18, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Chain involving expired intermediary', VerificationResults:Status == [verified, expired, verified, verified]:true):-
+test('Chain involving expired intermediary', VerificationResults:Status == [expired, verified]:true):-
     do_verification_test(20, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Chain involving not-yet-valid intermediary', VerificationResults:Status == [verified, not_yet_valid, verified, verified]:true):-
+test('Chain involving not-yet-valid intermediary', VerificationResults:Status == [not_yet_valid, verified]:true):-
     do_verification_test(21, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Chain involving intermediary not authorized to sign certificates', VerificationResults:Status == [invalid_ca, bad_certificate_use, verified, verified, verified]:true):-
+test('Chain involving intermediary not authorized to sign certificates', VerificationResults:Status == [bad_certificate_use, invalid_ca, verified]:true):-
     do_verification_test(22, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Confirm that a failure in the verification callback triggers a connection abort', Status = error(_)):-
@@ -657,28 +678,28 @@ test('Confirm that an exception in the verification callback triggers a connecti
     do_verification_test(17, try_ssl_client('www.example.com', abort_verify_hook), _, Status).
 
 
-test('Certificate has a CRL but has not been revoked. We do not provide the CRL', VerificationResults:Status == [unknown_crl, unknown_crl, verified, verified]:true):-
+test('Certificate has a CRL but has not been revoked. We do not provide the CRL', VerificationResults:Status == [unknown_crl, verified]:true):-
     do_verification_test(23, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true)]), VerificationResults, Status).
 
-test('Certificate has a CRL but has not been revoked. We do provide the CRL', VerificationResults:Status == [verified, verified]:true):-
+test('Certificate has a CRL but has not been revoked. We do provide the CRL', VerificationResults:Status == [verified]:true):-
     do_verification_test(23, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true), crl(['tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
-test('Certificate has a CRL and has been revoked. We do not provide the CRL', VerificationResults:Status == [unknown_crl, unknown_crl, verified, verified]:true):-
+test('Certificate has a CRL and has been revoked. We do not provide the CRL', VerificationResults:Status == [unknown_crl, verified]:true):-
     do_verification_test(24, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true)]), VerificationResults, Status).
 
-test('Certificate has a CRL and has been revoked. We do provide the CRL', VerificationResults:Status == [revoked, verified, verified]:true):-
+test('Certificate has a CRL and has been revoked. We do provide the CRL', VerificationResults:Status == [revoked, verified]:true):-
     do_verification_test(24, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true), crl(['tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
-test('Certificate has a CRL but we want to ignore it', VerificationResults:Status == [verified, verified]:true):-
+test('Certificate has a CRL but we want to ignore it', VerificationResults:Status == [verified]:true):-
     do_verification_test(24, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
-test('Certificate has an illegal CRL', VerificationResults:Status == [bad_certificate_use, verified, verified, verified]:true):-
+test('Certificate has an illegal CRL', VerificationResults:Status == [bad_certificate_use, verified]:true):-
     do_verification_test(25, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true), crl(['tests/test_certs/25-crl.pem', 'tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
-test('Intermediate CA has revoked the certificate', VerificationResults:Status == [revoked, verified, verified, verified]:true):-
+test('Intermediate CA has revoked the certificate', VerificationResults:Status == [revoked, verified]:true):-
     do_verification_test(26, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true), crl(['tests/test_certs/26-crl.pem', 'tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
-test('root CA has revoked the intermediate CA', VerificationResults:Status == [revoked, verified, verified, verified]:true):-
+test('root CA has revoked the intermediate CA', VerificationResults:Status == [revoked, verified]:true):-
     do_verification_test(27, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true), crl(['tests/test_certs/27-crl.pem', 'tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
 
@@ -710,9 +731,9 @@ test(roundtrip, RecoveredText == Text) :-
 :- end_tests(crypto_data_encrypt).
 
 
-                 /*******************************
-                 *             UTIL             *
-                 *******************************/
+		 /*******************************
+		 *             UTIL             *
+		 *******************************/
 
 is_certificate(Cert) :-
     blob(Cert, ssl_certificate),
@@ -768,5 +789,3 @@ is_hex_string(S) :-
 is_hex(C) :- between(0'0, 0'9, C), !.
 is_hex(C) :- between(0'A, 0'F, C), !.
 is_hex(C) :- between(0'a, 0'f, C), !.
-
-
