@@ -496,7 +496,11 @@ linkValI(DECL_LD Word p)
 #define is_signalled(_) LDFUNC(is_signalled, _)
 static inline int
 is_signalled(DECL_LD)
-{ return HAS_LD && unlikely((LD->signal.pending[0]|LD->signal.pending[1]) != 0);
+{ if (!HAS_LD) return FALSE;
+  for (int i = 0; i < SIGMASK_WORDS; i++)
+  { if (unlikely(LD->signal.pending[i] != 0)) return TRUE;
+  }
+  return FALSE;
 }
 
 #define register_attvar(gp) LDFUNC(register_attvar, gp)
@@ -532,7 +536,7 @@ visibleClause(DECL_LD Clause cl, gen_t gen)
   if ( c <= gen && e > gen )
     return TRUE;
 
-  if ( unlikely(gen >= LD->transaction.gen_base) &&
+  if ( unlikely(LD->transaction.gen_base && gen >= LD->transaction.gen_base) &&
        true(cl->predicate, P_DYNAMIC) )
     return transaction_visible_clause(cl, gen);
 
@@ -617,6 +621,11 @@ setGenerationFrame(DECL_LD LocalFrame fr)
   }
 }
 
+static inline QueryFrame
+QueryFromQid(QueryRef qid)
+{ return (QueryFrame) (&((Word)qid->engine->stacks.local.base)[qid->offset]);
+}
+
 		 /*******************************
 		 *	      INDEXING		*
 		 *******************************/
@@ -631,20 +640,24 @@ The hash should not conflict  with   a  functor_t  (hence the STG_GLOBAL
 mask) and may never be 0.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+#define KEY_INDEX_MAX 4
+
 static inline word
-murmur_key(void *ptr, size_t n)
+murmur_key(const void *ptr, size_t n)
 { word k;
 
   DEBUG(0, assert(n%sizeof(word) == 0));
 
-  if ( n > sizeof(word)*4 )
-  { word data[3];
+  if ( n > sizeof(word)*KEY_INDEX_MAX )
+  { word data[KEY_INDEX_MAX];
+    const Word in = (const Word)ptr;
 
-    data[0] = ((Word)ptr)[0];
-    data[1] = ((Word)ptr)[n/sizeof(word)-1];
-    data[2] = n;
+    for(size_t i=0; i<KEY_INDEX_MAX-2; i++)
+      data[i] = in[i];
+    data[KEY_INDEX_MAX-2] = in[n/sizeof(word)-1];
+    data[KEY_INDEX_MAX-1] = n;
 
-    k = MurmurHashAligned2(data, sizeof(word)*3, MURMUR_SEED);
+    k = MurmurHashAligned2(data, sizeof(word)*4, MURMUR_SEED);
   } else
   { k = MurmurHashAligned2(ptr, n, MURMUR_SEED);
   }
